(ns bombnet.manager
  (:require [bombnet.game :refer :all]
            [clojure.set :refer [difference]]))

(defrecord Room [clients players state])

(defrecord Manager [rooms])

(defn new-manager []
  (->Manager (atom {})))

(defn ^:private get-room [rooms room]
  (if-let [found-room (get @rooms room)]
    found-room
    (let
      [new-room (->Room {} #{} nil)]
      (swap! rooms assoc room new-room)
      new-room)))

(defmacro defn-room [name params & body]
  (let [actual-params (vec (concat ['{:keys [rooms]} 'room-id]
                                   params))
        actual-body `(let [~'room (get-room ~'rooms ~'room-id)
                           ~'update-room #(swap! ~'rooms assoc ~'room-id %)]
                       ~@body)]
    `(defn ~name ~actual-params
       ~actual-body)))

(defn-room client-join [id channel]
  (update-room (assoc-in room [:clients id] {:id id :channel channel}))
  true)

(defn-room client-leave [id]
  (let [new-room (-> room
                     (update :clients dissoc id)
                     (update :players difference #{id}))]
    (update-room (if (not-empty (:clients new-room)) new-room))
    true))

(defn-room player-join [id]
  (let [player-count (count (:players room))]
    (if (>= player-count 4)
      false
      (do
        (update-room (update room :players conj id))
        true))))

(defn-room player-leave [id]
  (update-room (update room :players difference #{id}))
  true)

(defn-room player-action [id action]
  (update-room (update room :state bombnet.game/queue-action action id)))

(defn-room start-game []
  (let [players (filter (:players room) (:clients room))
        new-state (new-game players)]
    (update-room (assoc room :state new-state))))

(defn room-status [{:keys [rooms]} room-id]
  (get @rooms room-id))
